SpeedControl = {};
SpeedControl.visibleEventText = false;
SpeedControl.visibleOnDraw = true;

local modDir = g_currentModDirectory;

function SpeedControl.initSpecialization()

    local schemaSavegame = Vehicle.xmlSchemaSavegame
    schemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?).FS25_VehicleInspector.SpeedControl#speed1", "Cruise control speed 1")
    schemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?).FS25_VehicleInspector.SpeedControl#speed2", "Cruise control speed 2")
    schemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?).FS25_VehicleInspector.SpeedControl#speed3", "Cruise control speed 3")
    schemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?).FS25_VehicleInspector.SpeedControl#currentKey", "Current speed index")

end;

function SpeedControl.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Enterable, specializations) and SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function SpeedControl.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "setSpeedControlKey",            	SpeedControl.setSpeedControlKey)
    SpecializationUtil.registerFunction(vehicleType, "setSpeedControl",            		SpeedControl.setSpeedControl)
    --SpecializationUtil.registerFunction(vehicleType, "MPEvent",            				SpeedControl.MPEvent)
end

function SpeedControl.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad"
		,"onPostLoad"		
		,"saveToXMLFile"
		,"onRegisterActionEvents"
		,"onReadStream"
		,"onWriteStream"
		,"onUpdate"		
		,"onDraw"
	};	
	for i, v in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, v, SpeedControl);
	end;
end;

function SpeedControl:onLoad(savegame)	
	self.setSpeedControlKey = SpeedControl.setSpeedControlKey;
	self.setSpeedControl = SpeedControl.setSpeedControl;
	self.setCruiseControlState = Utils.appendedFunction(Drivable.setCruiseControlState, SpeedControl.setCruiseControlState);	
	self.speedControl = {};
	self.spec_speedControl = self.speedControl;
	self.speedControl.KeyPosX = 0;
	self.speedControl.KeyPosY = 0;
	self.speedControl.KeyTextSize = 1;	
	self.speedControl.actionEvents = {};
	self.speedControl.inputValues = {};
	self.speedControl.inputValues.axisSteer = 0;
	self.speedControl.actionEventIds = {};	

	self.speedControl.keys = {};
	self.speedControl.keys[1] = {};
	self.speedControl.keys[1].speed = 10;
	self.speedControl.keys[1].isActive = false;
	self.speedControl.keys[1].isSelected = false;

	self.speedControl.keys[2] = {};
	self.speedControl.keys[2].speed = 20;
	self.speedControl.keys[2].isActive = false;
	self.speedControl.keys[2].isSelected = false;

	self.speedControl.keys[3] = {};
	local motor = self:getMotor();
	self.speedControl.keys[3].speed = math.ceil(motor:getMaximumForwardSpeed()*3.6);
	self.speedControl.keys[3].isActive = false;
	self.speedControl.keys[3].isSelected = true;

	self.speedControl.currentKey = 3;	
	self.speedControl.changeWaitTime = 0;	
end;

function SpeedControl:onPostLoad(savegame)	
	if savegame ~= nil then
		local val1 = savegame.xmlFile:getValue(savegame.key..".FS25_VehicleInspector.SpeedControl#speed1")
		if val1 ~= nil then
			self.speedControl.keys[1].speed = val1;
		end;
		local val2 = savegame.xmlFile:getValue(savegame.key..".FS25_VehicleInspector.SpeedControl#speed2")
		if val2~= nil then
			self.speedControl.keys[2].speed = val2;
		end;
		local val3 = savegame.xmlFile:getValue(savegame.key..".FS25_VehicleInspector.SpeedControl#speed3")
		if val3 ~= nil then
			self.speedControl.keys[3].speed = val3;
		end;
		local val4 = savegame.xmlFile:getValue(savegame.key..".FS25_VehicleInspector.SpeedControl#currentKey")
		if val4 ~= nil then
			self.speedControl.currentKey = val4;
			self.speedControl.keys[val4].isSelected = true;
		end;
	end;	
	if self.spec_drivable.cruiseControl ~= nil then	
		self.spec_drivable.cruiseControl.isActive = false;
		self.spec_drivable.cruiseControl.speed = self.speedControl.keys[self.speedControl.currentKey].speed;		
	end;	
end;

function SpeedControl:saveToXMLFile(xmlFile, key)
	xmlFile:setValue(key.."#speed1",    math.ceil(self.speedControl.keys[1].speed));
	xmlFile:setValue(key.."#speed2",    math.ceil(self.speedControl.keys[2].speed));
	xmlFile:setValue(key.."#speed3",    math.ceil(self.speedControl.keys[3].speed));
	xmlFile:setValue(key.."#currentKey", self.speedControl.currentKey);
end;

function SpeedControl:setVisibleEventText(onStart)
	if self.isClient then
		local spec = self.speedControl;
		if spec ~= nil and spec.actionEventIds ~= nil then
			if onStart == nil then SpeedControl.visibleEventText = not SpeedControl.visibleEventText;end;
			for _, actionEventId in ipairs(spec.actionEventIds) do
				g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
			end;
		end;
	end;
end;

function SpeedControl:onRegisterActionEvents(isActiveForInput)
    if self.isClient then
		local spec = self.speedControl;
		self:clearActionEventsTable(spec.actionEvents);
		spec.actionEventIds = {};			
		if self:getIsActiveForInput(true, true) then
            local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED1", self, SpeedControl.onInputAction, false, true, false, true, nil);
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
            g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
			table.insert(spec.actionEventIds, actionEventId);
			
			if actionEventId == "" then
				local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED1", self, SpeedControl.onInputAction, false, true, false, true, nil);
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
				g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
				table.insert(spec.actionEventIds, actionEventId);
			end;
			
			local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED2", self, SpeedControl.onInputAction, false, true, false, true, nil);
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
            g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
			table.insert(spec.actionEventIds, actionEventId);
			
			if actionEventId == "" then
				local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED2", self, SpeedControl.onInputAction, false, true, false, true, nil);
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
				g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
				table.insert(spec.actionEventIds, actionEventId);
			end;
			
			local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED3", self, SpeedControl.onInputAction, false, true, false, true, nil);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
			g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
			table.insert(spec.actionEventIds, actionEventId);
			
			if actionEventId == "" then
				local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_SPEED3", self, SpeedControl.onInputAction, false, true, false, true, nil);
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
				g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
				table.insert(spec.actionEventIds, actionEventId);
			end;
			
			local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_AXIS_SETSPEED", self, SpeedControl.onInputAction_setSpeed, false, false, true, true, nil);			
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
            g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
			table.insert(spec.actionEventIds, actionEventId);
			
			if actionEventId == "" then
				local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_AXIS_SETSPEED", self, SpeedControl.onInputAction_setSpeed, false, false, true, true, nil);			
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW);
				g_inputBinding:setActionEventTextVisibility(actionEventId, SpeedControl.visibleEventText);
				table.insert(spec.actionEventIds, actionEventId);
			end;
			
			local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_AXIS_SETSPEED", self, SpeedControl.onInputAction_setSpeedEvent, false, true, false, true, nil);
			table.insert(spec.actionEventIds, actionEventId);
			
			if actionEventId == "" then
				local _, actionEventId = self:addActionEvent(spec.actionEvents, "VI_SPEEDCONTROL_AXIS_SETSPEED", self, SpeedControl.onInputAction_setSpeedEvent, false, true, false, true, nil);
				table.insert(spec.actionEventIds, actionEventId);
			end;
        end;
    end;
end;

function SpeedControl.onInputAction(self, actionName, inputValue, callbackState, isAnalog)
	if actionName == "VI_SPEEDCONTROL_SPEED1" then
		self.speedControl.currentKey = 1;
		self:setSpeedControlKey(1);
		SpeedControlMPEvent.updateSendEvent(self);
	end;
	if actionName == "VI_SPEEDCONTROL_SPEED2" then
		self.speedControl.currentKey = 2;
		self:setSpeedControlKey(2);
		SpeedControlMPEvent.updateSendEvent(self);
	end;
	if actionName == "VI_SPEEDCONTROL_SPEED3" then
		self.speedControl.currentKey = 3;
		self:setSpeedControlKey(3);
		SpeedControlMPEvent.updateSendEvent(self);
	end;	
	self.spec_drivable.cruiseControl.speed = math.ceil(self.speedControl.keys[self.speedControl.currentKey].speed);
	SpeedControlSpeedMPEvent.updateSendEvent(self);	
end;

function SpeedControl.onInputAction_setSpeed(self, actionName, inputValue, callbackState, isAnalog)
    if self:getIsEntered() then
        self.speedControl.inputValues.axisSteer = inputValue;
    end;	
end;

function SpeedControl.onInputAction_setSpeedEvent(self, actionName, inputValue, callbackState, isAnalog)
    if self:getIsEntered() then		
		local useReverseSpeed = self:getDrivingDirection() < 0;
		local modSpec = self.spec_extendedCruiseControl;
		local permanentActive = false;
		if modSpec ~= nil and modSpec.permanentActive then permanentActive = true;end;
		if self:getReverserDirection() < 0 then
			useReverseSpeed = not useReverseSpeed;
		end;
		if useReverseSpeed then
			if not permanentActive then
				local speedReverse = self.spec_drivable.cruiseControl.speedReverse;
				if inputValue < 0 and self.spec_drivable.cruiseControl.speedReverse > 1 then
					self.spec_drivable.cruiseControl.speedReverse = self.spec_drivable.cruiseControl.speedReverse - 1;
				elseif inputValue > 0 and self.spec_drivable.cruiseControl.speedReverse < self.spec_drivable.cruiseControl.maxSpeedReverse then
					self.spec_drivable.cruiseControl.speedReverse = self.spec_drivable.cruiseControl.speedReverse + 1;
				end;
				self:setCruiseControlMaxSpeed(nil, self.spec_drivable.cruiseControl.speedReverse);
				SpeedControlReverseSpeedMPEvent.updateSendEvent(self);
			end;
		else
			if inputValue < 0 and self.speedControl.keys[self.speedControl.currentKey].speed > 1 then
				self.speedControl.keys[self.speedControl.currentKey].speed = self.speedControl.keys[self.speedControl.currentKey].speed - 1;
			elseif inputValue > 0 and self.speedControl.keys[self.speedControl.currentKey].speed < math.ceil(self:getMotor():getMaximumForwardSpeed()*3.6) then
				self.speedControl.keys[self.speedControl.currentKey].speed = self.speedControl.keys[self.speedControl.currentKey].speed + 1;
			end;
			self.spec_drivable.cruiseControl.speed = math.ceil(self.speedControl.keys[self.speedControl.currentKey].speed);
			
			SpeedControlSpeedMPEvent.updateSendEvent(self);	
		end;	
	end;	
end;

function SpeedControl:onReadStream(streamId, connection)
    if connection:getIsServer() then
        self.speedControl.currentKey = streamReadInt8(streamId);

		self.speedControl.keys[1].isSelected = streamReadBool(streamId);
		self.speedControl.keys[1].isActive = streamReadBool(streamId);
		self.speedControl.keys[1].speed = streamReadInt8(streamId);
		
		self.speedControl.keys[2].isSelected = streamReadBool(streamId);
		self.speedControl.keys[2].isActive = streamReadBool(streamId);
		self.speedControl.keys[2].speed = streamReadInt8(streamId);
		
		self.speedControl.keys[3].isSelected = streamReadBool(streamId);
		self.speedControl.keys[3].isActive = streamReadBool(streamId);
		self.speedControl.keys[3].speed = streamReadInt8(streamId);
		
		self.spec_drivable.cruiseControl.speed = math.ceil(self.speedControl.keys[self.speedControl.currentKey].speed);
    end;
end;

function SpeedControl:onWriteStream(streamId, connection)
    if not connection:getIsServer() then
        streamWriteInt8(streamId, self.speedControl.currentKey);
		
		streamWriteBool(streamId, self.speedControl.keys[1].isSelected);
		streamWriteBool(streamId, self.speedControl.keys[1].isActive);
		streamWriteInt8(streamId, self.speedControl.keys[1].speed);
		
		streamWriteBool(streamId, self.speedControl.keys[2].isSelected);
		streamWriteBool(streamId, self.speedControl.keys[2].isActive);
		streamWriteInt8(streamId, self.speedControl.keys[2].speed);
		
		streamWriteBool(streamId, self.speedControl.keys[3].isSelected);
		streamWriteBool(streamId, self.speedControl.keys[3].isActive);
		streamWriteInt8(streamId, self.speedControl.keys[3].speed);
    end;
end;

function SpeedControl:onUpdate(dt)
	local spec = self.spec_enterable;
	local modSpec = self.spec_extendedCruiseControl;
	if spec.isEntered and self.isClient then		
		local useReverseSpeed = self:getDrivingDirection() < 0;
		local permanentActive = false;
		if modSpec ~= nil and modSpec.permanentActive then permanentActive = true;end;
		if self:getReverserDirection() < 0 then
			useReverseSpeed = not useReverseSpeed;			
		end;
		if useReverseSpeed then
			if not permanentActive then
				local speedReverse = self.spec_drivable.cruiseControl.speedReverse;			
				if self.speedControl.inputValues.axisSteer < 0 and self.spec_drivable.cruiseControl.speedReverse > 1 then
					self.speedControl.changeWaitTime = self.speedControl.changeWaitTime + dt;
					if self.speedControl.changeWaitTime > 250 then
						self.spec_drivable.cruiseControl.speedReverse = self.spec_drivable.cruiseControl.speedReverse - 15*dt/1000;
					end;
					if self.spec_drivable.cruiseControl.speedReverse < 1 then
						self.spec_drivable.cruiseControl.speedReverse = 1;
					end;
				elseif self.speedControl.inputValues.axisSteer > 0 and self.spec_drivable.cruiseControl.speedReverse < self.spec_drivable.cruiseControl.maxSpeedReverse then				
					self.speedControl.changeWaitTime = self.speedControl.changeWaitTime + dt;
					if self.speedControl.changeWaitTime > 250 then
						self.spec_drivable.cruiseControl.speedReverse = self.spec_drivable.cruiseControl.speedReverse + 15*dt/1000;
					end;				
					if self.spec_drivable.cruiseControl.speedReverse > self.spec_drivable.cruiseControl.maxSpeedReverse then
						self.spec_drivable.cruiseControl.speedReverse = self.spec_drivable.cruiseControl.maxSpeedReverse;
					end;
				else
					self.speedControl.changeWaitTime = 0;
				end;						
				if self.spec_drivable.cruiseControl.speedReverse ~= speedReverse then
					self:setCruiseControlMaxSpeed(nil, self.spec_drivable.cruiseControl.speedReverse);
					SpeedControlReverseSpeedMPEvent.updateSendEvent(self);	
				end;
			end;
		else			
			local currSpeed = self.speedControl.keys[self.speedControl.currentKey].speed;			
			if self.speedControl.inputValues.axisSteer < 0 and self.speedControl.keys[self.speedControl.currentKey].speed > 1 then
				self.speedControl.changeWaitTime = self.speedControl.changeWaitTime + dt;
				if self.speedControl.changeWaitTime > 250 then
					self.speedControl.keys[self.speedControl.currentKey].speed = self.speedControl.keys[self.speedControl.currentKey].speed - 15*dt/1000;
				end;
				if self.speedControl.keys[self.speedControl.currentKey].speed < 1 then
					self.speedControl.keys[self.speedControl.currentKey].speed = 1;
				end;
			elseif self.speedControl.inputValues.axisSteer > 0 and self.speedControl.keys[self.speedControl.currentKey].speed < math.ceil(self:getMotor():getMaximumForwardSpeed()*3.6) then				
				self.speedControl.changeWaitTime = self.speedControl.changeWaitTime + dt;
				if self.speedControl.changeWaitTime > 250 then
					self.speedControl.keys[self.speedControl.currentKey].speed = self.speedControl.keys[self.speedControl.currentKey].speed + 15*dt/1000;
				end;				
				if self.speedControl.keys[self.speedControl.currentKey].speed > math.ceil(self:getMotor():getMaximumForwardSpeed()*3.6) then
					self.speedControl.keys[self.speedControl.currentKey].speed = math.ceil(self:getMotor():getMaximumForwardSpeed()*3.6);
				end;
			else
				self.speedControl.changeWaitTime = 0;
			end;			
			if self.speedControl.keys[self.speedControl.currentKey].speed ~= currSpeed then		
				self.spec_drivable.cruiseControl.speed = math.ceil(self.speedControl.keys[self.speedControl.currentKey].speed);
				SpeedControlSpeedMPEvent.updateSendEvent(self);				
			end;			
		end;			
	end;
end;

function SpeedControl:onDraw()
	if self.spec_enterable.isEntered and SpeedControl.visibleOnDraw then		
		local cruiseOverlay = g_currentMission.hud.speedMeter.cruiseControl;	
		if cruiseOverlay ~= nil then
			self.speedControl.KeyPosX = cruiseOverlay.x + cruiseOverlay.width * 0.85;
			self.speedControl.KeyPosY = cruiseOverlay.y + cruiseOverlay.height * 0.65;
			self.speedControl.KeyTextSize = g_currentMission.hud.speedMeter.speedUnitTextSize * 0.6;
			if self.spec_drivable.cruiseControl.state == 1 then
				setTextColor(1, 1, 0, 1);			
			else
				setTextColor(1,1,1,1);
			end;
			setTextBold(true);
			setTextAlignment(RenderText.ALIGN_LEFT);
			renderText(self.speedControl.KeyPosX, self.speedControl.KeyPosY, self.speedControl.KeyTextSize, tostring(self.speedControl.currentKey));		
			setTextColor(1,1,1,1);
			setTextBold(false);
			setTextAlignment(0);
		end;
	end;
end;

function SpeedControl:setSpeedControlKey(key)
	if self.speedControl.keys[key].isActive then
		for i=1, table.getn(self.speedControl.keys) do
			self.speedControl.keys[i].isActive = false;	
			self.speedControl.keys[i].isSelected = false;	
		end;
		self.speedControl.keys[key].isSelected = true;
		self:setSpeedControl(key, false);
	else
		for i=1, table.getn(self.speedControl.keys) do
			self.speedControl.keys[i].isActive = false;	
			self.speedControl.keys[i].isSelected = false;	
		end;
		self.speedControl.keys[key].isActive = true;	
		self.speedControl.keys[key].isSelected = true;
		self:setSpeedControl(key, true);
	end;
end;

function SpeedControl:setSpeedControl(key, active)
	if active then
		self.spec_drivable.cruiseControl.state = 1;
		self:getMotor():setSpeedLimit(self.spec_drivable.cruiseControl.speed);
	else
		self.spec_drivable.cruiseControl.state = 0;
		self:getMotor():setSpeedLimit(math.huge);
	end;
end;

function SpeedControl:setCruiseControlState(state, noEventSend)
	for i=1, table.getn(self.speedControl.keys) do
		self.speedControl.keys[i].isActive = false;	
	end;	
end;


